"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/* global describe, it, before, afterEach */
const assert = require("assert");
const http = require("http");
const path = require("path");
const express = require("express");
const node_fetch_1 = require("node-fetch");
const src_1 = require("../src");
const PORT = 3030;
const GET = (url, obj) => __awaiter(void 0, void 0, void 0, function* () {
    const res = yield node_fetch_1.default(`http://localhost:${PORT}${url}`, { method: 'GET' });
    assert.deepStrictEqual(yield res.json(), obj);
    return res;
});
const POST = (url, body, obj) => __awaiter(void 0, void 0, void 0, function* () { return assert.deepStrictEqual(yield node_fetch_1.default(`http://localhost:${PORT}${url}`, { method: 'POST', body }).then(res => res.json()), obj); });
const PUT = (url, body, obj) => __awaiter(void 0, void 0, void 0, function* () { return assert.deepStrictEqual(yield node_fetch_1.default(`http://localhost:${PORT}${url}`, { method: 'PUT', body }).then(res => res.json()), obj); });
const DELETE = (url, obj) => __awaiter(void 0, void 0, void 0, function* () { return assert.deepStrictEqual(yield node_fetch_1.default(`http://localhost:${PORT}${url}`, { method: 'DELETE' }).then(res => res.json()), obj); });
const ERROR_RESPONSE = { code: 400, status: 'error', message: 'Method Not Implemented' };
const app = express();
const api = src_1.default(express, {
    root: path.join(__dirname, 'fixtures', 'discovery'),
});
app.use('/api', api);
// Start Server
const server = http.createServer(app).listen(PORT, function () {
    console.log(('Express server listening on port ' + app.get('port')));
});
describe('API Discovery', function () {
    describe('it should', function () {
        this.afterAll(() => {
            server.close();
        });
        it('work with default exports', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield GET('/api', { status: 'success', data: false });
            });
        });
        it('default export instances retain state', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield GET('/api', { status: 'success', data: true });
            });
        });
        it('allows custom response codes', function () {
            return __awaiter(this, void 0, void 0, function* () {
                const res = yield GET('/api/status', { status: 'error', message: 'Unauthorized.' });
                assert.strictEqual(res.status, 401);
            });
        });
        it('work with named GET exports', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield GET('/api/named', { status: 'success', data: 'get' });
            });
        });
        it('work with named POST exports', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield POST('/api/named', {}, { status: 'success', data: 'post' });
            });
        });
        it('work with named PUT exports', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield PUT('/api/named', {}, { status: 'success', data: 'put' });
            });
        });
        it('work with named DELETE exports', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield DELETE('/api/named', { status: 'success', data: 'delete' });
            });
        });
        it('work with path patterns', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield GET('/api/pathpatterns', { status: 'success', data: 'Path Patterns' });
                yield GET('/api/pathptterns', { status: 'success', data: 'Path Patterns' });
            });
        });
        it('work with named required params', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield GET('/api/required-params/what', { dynamic: 'what' });
                yield GET('/api/required-params', ERROR_RESPONSE);
            });
        });
        it('work with named optional params', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield GET('/api/optional-params/what', { dynamic: 'what' });
                yield GET('/api/optional-params', {});
            });
        });
        it('work with named required params – windows', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield GET('/api/required-params/win/what', { dynamic: 'what' });
                yield GET('/api/required-params', ERROR_RESPONSE);
            });
        });
        it('work with named optional params – windows', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield GET('/api/optional-params/win/what', { dynamic: 'what' });
                yield GET('/api/optional-params', {});
            });
        });
    });
});
//# sourceMappingURL=discovery.test.js.map